"""
Constants.

Constants for feeds and feed entries.
"""

ATTR_ATTRIBUTION = 'attribution'
CUSTOM_ATTRIBUTE = 'custom_attribute'

XML_ATTR_HREF = '@href'
XML_ATTR_TERM = '@term'

XML_CDATA = '#text'

XML_TAG_AUTHOR = 'author'
XML_TAG_CATEGORY = 'category'
XML_TAG_CHANNEL = 'channel'
XML_TAG_CONTENT = 'content'
XML_TAG_CONTRIBUTOR = 'contributor'
XML_TAG_COPYRIGHT = 'copyright'
XML_TAG_DC_DATE = 'dc:date'
XML_TAG_DESCRIPTION = 'description'
XML_TAG_DOCS = 'docs'
XML_TAG_ENTRY = 'entry'
XML_TAG_FEED = 'feed'
XML_TAG_GENERATOR = 'generator'
XML_TAG_GEO_LAT = 'geo:lat'
XML_TAG_GEO_LONG = 'geo:long'
XML_TAG_GEO_POINT = 'geo:Point'
XML_TAG_GEORSS_POINT = 'georss:point'
XML_TAG_GEORSS_POLYGON = 'georss:polygon'
XML_TAG_GEORSS_WHERE = 'georss:where'
XML_TAG_GML_EXTERIOR = 'gml:exterior'
XML_TAG_GML_LINEAR_RING = 'gml:LinearRing'
XML_TAG_GML_POINT = 'gml:Point'
XML_TAG_GML_POLYGON = 'gml:Polygon'
XML_TAG_GML_POS = 'gml:pos'
XML_TAG_GML_POS_LIST = 'gml:posList'
XML_TAG_GUID = 'guid'
XML_TAG_HEIGHT = 'height'
XML_TAG_ID = 'id'
XML_TAG_IMAGE = 'image'
XML_TAG_ITEM = 'item'
XML_TAG_LANGUAGE = 'language'
XML_TAG_LAST_BUILD_DATE = 'lastBuildDate'
XML_TAG_LINK = 'link'
XML_TAG_MANAGING_EDITOR = 'managingEditor'
XML_TAG_NAME = 'name'
XML_TAG_PUB_DATE = 'pubDate'
XML_TAG_PUBLISHED = 'published'
XML_TAG_RIGHTS = 'rights'
XML_TAG_RSS = 'rss'
XML_TAG_SOURCE = 'source'
XML_TAG_SUBTITLE = 'subtitle'
XML_TAG_SUMMARY = 'summary'
XML_TAG_TITLE = 'title'
XML_TAG_TTL = 'ttl'
XML_TAG_UPDATED = 'updated'
XML_TAG_URL = 'url'
XML_TAG_WIDTH = 'width'
